@@ -115,7 +115,7 @@ xplr.config.general.prompt.style = {}
 -- Type: nullable string
 xplr.config.general.logs.info.format = "INFO"
 
--- The style for the informations logs.
+-- The style for the information logs.
 --
 -- Type: [Style](https://xplr.dev/en/style)
 xplr.config.general.logs.info.style = { fg = "LightBlue" }
@@ -157,11 +157,11 @@ xplr.config.general.logs.error.style = { fg = "Red" }
 -- * format: nullable string
 -- * style: [Style](https://xplr.dev/en/style)
 xplr.config.general.table.header.cols = {
-  { format = " index", style = {} },
+  { format = " index",            style = {} },
   { format = "╭─── path", style = {} },
-  { format = "permissions", style = {} },
-  { format = "size", style = {} },
-  { format = "modified", style = {} },
+  { format = "perm",              style = {} },
+  { format = "size",              style = {} },
+  { format = "modified",          style = {} },
 }
 
 -- Style of the table header.
@@ -246,6 +246,26 @@ xplr.config.general.table.col_widths = {
   { Percentage = 20 },
 }
 
+-- Renderer for each item in the selection list.
+--
+-- Type: nullable string
+xplr.config.general.selection.item.format = "builtin.fmt_general_selection_item"
+
+-- Style for each item in the selection list.
+--
+-- Type: [Style](https://xplr.dev/en/style)
+xplr.config.general.selection.item.style = {}
+
+-- The default search algorithm
+--
+-- Type: [Search Algorithm](https://xplr.dev/en/searching#algorithm)
+xplr.config.general.search.algorithm = "Fuzzy"
+
+-- The default search ordering
+--
+-- Type: boolean
+xplr.config.general.search.unordered = false
+
 -- The content that is placed before the item name for each row by default.
 --
 -- Type: nullable string
@@ -414,7 +434,6 @@ xplr.config.general.sort_and_filter_ui.filter_identifiers = {
   RelativePathIsNot = { format = "rel!=", style = {} },
   RelativePathDoesMatchRegex = { format = "rel=/", style = {} },
   RelativePathDoesNotMatchRegex = { format = "rel!/", style = {} },
-
   IRelativePathDoesContain = { format = "[i]rel=~", style = {} },
   IRelativePathDoesEndWith = { format = "[i]rel=$", style = {} },
   IRelativePathDoesNotContain = { format = "[i]rel!~", style = {} },
@@ -425,7 +444,6 @@ xplr.config.general.sort_and_filter_ui.filter_identifiers = {
   IRelativePathIsNot = { format = "[i]rel!=", style = {} },
   IRelativePathDoesMatchRegex = { format = "[i]rel=/", style = {} },
   IRelativePathDoesNotMatchRegex = { format = "[i]rel!/", style = {} },
-
   AbsolutePathDoesContain = { format = "abs=~", style = {} },
   AbsolutePathDoesEndWith = { format = "abs=$", style = {} },
   AbsolutePathDoesNotContain = { format = "abs!~", style = {} },
@@ -436,7 +454,6 @@ xplr.config.general.sort_and_filter_ui.filter_identifiers = {
   AbsolutePathIsNot = { format = "abs!=", style = {} },
   AbsolutePathDoesMatchRegex = { format = "abs=/", style = {} },
   AbsolutePathDoesNotMatchRegex = { format = "abs!/", style = {} },
-
   IAbsolutePathDoesContain = { format = "[i]abs=~", style = {} },
   IAbsolutePathDoesEndWith = { format = "[i]abs=$", style = {} },
   IAbsolutePathDoesNotContain = { format = "[i]abs!~", style = {} },
@@ -452,11 +469,22 @@ xplr.config.general.sort_and_filter_ui.filter_identifiers = {
 -- The identifiers used to denote applied search input.
 --
 -- Type: { format = nullable string, style = [Style](https://xplr.dev/en/style) }
-xplr.config.general.sort_and_filter_ui.search_identifier = {
-  format = "search:",
-  style = {},
+xplr.config.general.sort_and_filter_ui.search_identifiers = {
+  Fuzzy = { format = "fzy:", style = {} },
+  Regex = { format = "reg:", style = {} },
 }
 
+-- The shape of ordered indicator for search ordering identifiers in Sort & filter panel.
+--
+-- Type: nullable string
+xplr.config.general.sort_and_filter_ui.search_direction_identifiers.ordered.format =
+"↓"
+
+-- The shape of unordered indicator for search ordering identifiers in Sort & filter panel.
+--
+-- Type: nullable string
+xplr.config.general.sort_and_filter_ui.search_direction_identifiers.unordered.format = ""
+
 -- The content for panel title by default.
 --
 -- Type: nullable string
@@ -648,7 +676,7 @@ xplr.config.general.panel_ui.sort_and_filter.border_style = {}
 -- Type: nullable list of [Node Sorter](https://xplr.dev/en/sorting#node-sorter-applicable)
 xplr.config.general.initial_sorting = {
   { sorter = "ByCanonicalIsDir", reverse = true },
-  { sorter = "ByIRelativePath", reverse = false },
+  { sorter = "ByIRelativePath",  reverse = false },
 }
 
 -- The name of one of the modes to use when xplr loads.
@@ -700,9 +728,7 @@ xplr.config.general.global_key_bindings = {
 -- The style for the directory nodes
 --
 -- Type: [Style](https://xplr.dev/en/style)
-xplr.config.node_types.directory.style = {
-  fg = "Cyan",
-}
+xplr.config.node_types.directory.style = {}
 
 -- Metadata for the directory nodes.
 -- You can set as many metadata as you want.
@@ -738,10 +764,7 @@ xplr.config.node_types.file.meta.icon = "ƒ"
 -- The style for the symlink nodes.
 --
 -- Type: [Style](https://xplr.dev/en/style)
-xplr.config.node_types.symlink.style = {
-  fg = "Magenta",
-  add_modifiers = { "Italic" },
-}
+xplr.config.node_types.symlink.style = {}
 
 -- Metadata for the symlink nodes.
 -- You can set as many metadata as you want.
@@ -773,7 +796,7 @@ xplr.config.node_types.symlink.meta.icon = "§"
 -- xplr.config.node_types.mime_essence = {
 --   application = {
 --     -- application/*
---     ["*"] = { meta = { icon = "a" } }
+--     ["*"] = { meta = { icon = "a" } },
 --
 --     -- application/pdf
 --     pdf = { meta = { icon = "" }, style = { fg = "Blue" } },
@@ -1232,6 +1255,18 @@ xplr.config.modes.builtin.default = {
           "ScrollDownHalf",
         },
       },
+      ["ctrl-n"] = {
+        help = "next selection",
+        messages = {
+          "FocusNextSelection",
+        },
+      },
+      ["ctrl-p"] = {
+        help = "prev selection",
+        messages = {
+          "FocusPreviousSelection",
+        },
+      },
     },
     on_number = {
       help = "input",
@@ -1244,22 +1279,22 @@ xplr.config.modes.builtin.default = {
   },
 }
 
-xplr.config.modes.builtin.default.key_bindings.on_key["tab"] =
-  xplr.config.modes.builtin.default.key_bindings.on_key["ctrl-i"]
 xplr.config.modes.builtin.default.key_bindings.on_key["v"] =
-  xplr.config.modes.builtin.default.key_bindings.on_key["space"]
+    xplr.config.modes.builtin.default.key_bindings.on_key["space"]
 xplr.config.modes.builtin.default.key_bindings.on_key["V"] =
-  xplr.config.modes.builtin.default.key_bindings.on_key["ctrl-a"]
+    xplr.config.modes.builtin.default.key_bindings.on_key["ctrl-a"]
 xplr.config.modes.builtin.default.key_bindings.on_key["/"] =
-  xplr.config.modes.builtin.default.key_bindings.on_key["ctrl-f"]
+    xplr.config.modes.builtin.default.key_bindings.on_key["ctrl-f"]
 xplr.config.modes.builtin.default.key_bindings.on_key["h"] =
-  xplr.config.modes.builtin.default.key_bindings.on_key["left"]
+    xplr.config.modes.builtin.default.key_bindings.on_key["left"]
 xplr.config.modes.builtin.default.key_bindings.on_key["j"] =
-  xplr.config.modes.builtin.default.key_bindings.on_key["down"]
+    xplr.config.modes.builtin.default.key_bindings.on_key["down"]
 xplr.config.modes.builtin.default.key_bindings.on_key["k"] =
-  xplr.config.modes.builtin.default.key_bindings.on_key["up"]
+    xplr.config.modes.builtin.default.key_bindings.on_key["up"]
 xplr.config.modes.builtin.default.key_bindings.on_key["l"] =
-  xplr.config.modes.builtin.default.key_bindings.on_key["right"]
+    xplr.config.modes.builtin.default.key_bindings.on_key["right"]
+xplr.config.modes.builtin.default.key_bindings.on_key["tab"] =
+    xplr.config.modes.builtin.default.key_bindings.on_key["ctrl-i"] -- compatibility workaround
 
 -- The builtin debug error mode.
 --
@@ -1276,11 +1311,10 @@ xplr.config.modes.builtin.debug_error = {
       },
       splits = {
         {
-          CustomContent = {
-            title = "debug error",
-            body = {
-              StaticParagraph = {
-                render = [[
+          Static = {
+            CustomParagraph = {
+              ui = { title = { format = "debug error" } },
+              body = [[
 
   Some errors occurred during startup.
   If you think this is a bug, please report it at:
@@ -1292,8 +1326,7 @@ xplr.config.modes.builtin.debug_error = {
 
   To disable this mode, set `xplr.config.general.disable_debug_error_mode`
   to `true` in your config file.
-                ]],
-              },
+              ]],
             },
           },
         },
@@ -1332,11 +1365,10 @@ xplr.config.modes.builtin.debug_error = {
 xplr.config.modes.builtin.recover = {
   name = "recover",
   layout = {
-    CustomContent = {
-      title = " recover ",
-      body = {
-        StaticParagraph = {
-          render = [[
+    Static = {
+      CustomParagraph = {
+        ui = { title = { format = "recover" } },
+        body = [[
 
   You pressed an invalid key and went into "recover" mode.
   This mode saves you from performing unwanted actions.
@@ -1345,8 +1377,7 @@ xplr.config.modes.builtin.recover = {
 
   To disable this mode, set `xplr.config.general.enable_recover_mode`
   to `false` in your config file.
-          ]],
-        },
+        ]],
       },
     },
   },
@@ -1369,7 +1400,7 @@ xplr.config.modes.builtin.go_to_path = {
         messages = {
           {
             BashExecSilently0 = [===[
-              PTH=${XPLR_INPUT_BUFFER}
+              PTH="$XPLR_INPUT_BUFFER"
               PTH_ESC=$(printf %q "$PTH")
               if [ -d "$PTH" ]; then
                 "$XPLR" -m 'ChangeDirectory: %q' "$PTH"
@@ -1406,22 +1437,77 @@ xplr.config.modes.builtin.selection_ops = {
   layout = "HelpMenu",
   key_bindings = {
     on_key = {
+      ["e"] = {
+        help = "edit selection",
+        messages = {
+          {
+            BashExec0 = [===[
+              TMPFILE="$(mktemp)"
+              while IFS= read -r -d '' PTH; do
+                echo $(printf %q "${PTH:?}") >> "${TMPFILE:?}"
+              done < "${XPLR_PIPE_SELECTION_OUT:?}"
+              ${EDITOR:-vi} "${TMPFILE:?}"
+              [ ! -e "$TMPFILE" ] && exit
+              "$XPLR" -m ClearSelection
+              while IFS= read -r PTH_ESC; do
+                "$XPLR" -m 'SelectPath: %q' "$(eval printf %s ${PTH_ESC:?})"
+              done < "${TMPFILE:?}"
+              rm -- "${TMPFILE:?}"
+            ]===],
+          },
+          "PopMode",
+        },
+      },
+      ["l"] = {
+        help = "list selection",
+        messages = {
+          {
+            BashExec0 = [===[
+              [ -z "$PAGER" ] && PAGER="less -+F"
+
+              while IFS= read -r -d '' PTH; do
+                echo $(printf %q "$PTH")
+              done < "${XPLR_PIPE_SELECTION_OUT:?}" | ${PAGER:?}
+            ]===],
+          },
+          "PopMode",
+        },
+      },
       ["c"] = {
         help = "copy here",
         messages = {
           {
             BashExec0 = [===[
-              (while IFS= read -r -d '' PTH; do
+              "$XPLR" -m ExplorePwd
+              while IFS= read -r -d '' PTH; do
                 PTH_ESC=$(printf %q "$PTH")
-                if cp -vr -- "${PTH:?}" ./; then
-                  "$XPLR" -m 'LogSuccess: %q' "$PTH_ESC copied to ."
-                else
-                  "$XPLR" -m 'LogError: %q' "Failed to copy $PTH_ESC to ."
+                BASENAME=$(basename -- "$PTH")
+                BASENAME_ESC=$(printf %q "$BASENAME")
+                if [ -e "$BASENAME" ]; then
+                  echo
+                  echo "$BASENAME_ESC exists, do you want to overwrite it?"
+                  read -p "[y]es, [n]o, [S]kip: " ANS < /dev/tty
+                  case "$ANS" in
+                    [yY]*)
+                      ;;
+                    [nN]*)
+                      read -p "Enter new name: " BASENAME < /dev/tty
+                      BASENAME_ESC=$(printf %q "$BASENAME")
+                      ;;
+                    *)
+                      continue
+                      ;;
+                  esac
                 fi
-              done < "${XPLR_PIPE_SELECTION_OUT:?}")
-              "$XPLR" -m ExplorePwdAsync
-              "$XPLR" -m ClearSelection
-              read -p "[enter to continue]"
+                if cp -vr -- "${PTH:?}" "./${BASENAME:?}"; then
+                  "$XPLR" -m 'LogSuccess: %q' "$PTH_ESC copied to ./$BASENAME_ESC"
+                  "$XPLR" -m 'FocusPath: %q' "$BASENAME"
+                else
+                  "$XPLR" -m 'LogError: %q' "could not copy $PTH_ESC to ./$BASENAME_ESC"
+                fi
+              done < "${XPLR_PIPE_SELECTION_OUT:?}"
+              echo
+              read -p "[press enter to continue]"
             ]===],
           },
           "PopMode",
@@ -1432,16 +1518,116 @@ xplr.config.modes.builtin.selection_ops = {
         messages = {
           {
             BashExec0 = [===[
-              (while IFS= read -r -d '' PTH; do
+              "$XPLR" -m ExplorePwd
+              while IFS= read -r -d '' PTH; do
                 PTH_ESC=$(printf %q "$PTH")
-                if mv -v -- "${PTH:?}" ./; then
-                  "$XPLR" -m 'LogSuccess: %q' "$PTH_ESC moved to ."
-                else
-                  "$XPLR" -m 'LogError: %q' "Failed to move $PTH_ESC to ."
+                BASENAME=$(basename -- "$PTH")
+                BASENAME_ESC=$(printf %q "$BASENAME")
+                if [ -e "$BASENAME" ]; then
+                  echo
+                  echo "$BASENAME_ESC exists, do you want to overwrite it?"
+                  read -p "[y]es, [n]o, [S]kip: " ANS < /dev/tty
+                  case "$ANS" in
+                    [yY]*)
+                      ;;
+                    [nN]*)
+                      read -p "Enter new name: " BASENAME < /dev/tty
+                      BASENAME_ESC=$(printf %q "$BASENAME")
+                      ;;
+                    *)
+                      continue
+                      ;;
+                  esac
                 fi
-              done < "${XPLR_PIPE_SELECTION_OUT:?}")
-              "$XPLR" -m ExplorePwdAsync
-              read -p "[enter to continue]"
+                if mv -v -- "${PTH:?}" "./${BASENAME:?}"; then
+                  "$XPLR" -m 'LogSuccess: %q' "$PTH_ESC moved to ./$BASENAME_ESC"
+                  "$XPLR" -m 'FocusPath: %q' "$BASENAME"
+                else
+                  "$XPLR" -m 'LogError: %q' "could not move $PTH_ESC to ./$BASENAME_ESC"
+                fi
+              done < "${XPLR_PIPE_SELECTION_OUT:?}"
+              echo
+              read -p "[press enter to continue]"
+            ]===],
+          },
+          "PopMode",
+        },
+      },
+      ["s"] = {
+        help = "softlink here",
+        messages = {
+          {
+            BashExec0 = [===[
+              "$XPLR" -m ExplorePwd
+              while IFS= read -r -d '' PTH; do
+                PTH_ESC=$(printf %q "$PTH")
+                BASENAME=$(basename -- "$PTH")
+                BASENAME_ESC=$(printf %q "$BASENAME")
+                if [ -e "$BASENAME" ]; then
+                  echo
+                  echo "$BASENAME_ESC exists, do you want to overwrite it?"
+                  read -p "[y]es, [n]o, [S]kip: " ANS < /dev/tty
+                  case "$ANS" in
+                    [yY]*)
+                      ;;
+                    [nN]*)
+                      read -p "Enter new name: " BASENAME < /dev/tty
+                      BASENAME_ESC=$(printf %q "$BASENAME")
+                      ;;
+                    *)
+                      continue
+                      ;;
+                  esac
+                fi
+                if ln -sv -- "${PTH:?}" "./${BASENAME:?}"; then
+                  "$XPLR" -m 'LogSuccess: %q' "$PTH_ESC softlinked as ./$BASENAME_ESC"
+                  "$XPLR" -m 'FocusPath: %q' "$BASENAME"
+                else
+                  "$XPLR" -m 'LogError: %q' "could not softlink $PTH_ESC as ./$BASENAME_ESC"
+                fi
+              done < "${XPLR_PIPE_SELECTION_OUT:?}"
+              echo
+              read -p "[press enter to continue]"
+            ]===],
+          },
+          "PopMode",
+        },
+      },
+      ["h"] = {
+        help = "hardlink here",
+        messages = {
+          {
+            BashExec0 = [===[
+              "$XPLR" -m ExplorePwd
+              while IFS= read -r -d '' PTH; do
+                PTH_ESC=$(printf %q "$PTH")
+                BASENAME=$(basename -- "$PTH")
+                BASENAME_ESC=$(printf %q "$BASENAME")
+                if [ -e "$BASENAME" ]; then
+                  echo
+                  echo "$BASENAME_ESC exists, do you want to overwrite it?"
+                  read -p "[y]es, [n]o, [S]kip: " ANS < /dev/tty
+                  case "$ANS" in
+                    [yY]*)
+                      ;;
+                    [nN]*)
+                      read -p "Enter new name: " BASENAME < /dev/tty
+                      BASENAME_ESC=$(printf %q "$BASENAME")
+                      ;;
+                    *)
+                      continue
+                      ;;
+                  esac
+                fi
+                if ln -v -- "${PTH:?}" "./${BASENAME:?}"; then
+                  "$XPLR" -m 'LogSuccess: %q' "$PTH_ESC hardlinked as ./$BASENAME_ESC"
+                  "$XPLR" -m 'FocusPath: %q' "$BASENAME"
+                else
+                  "$XPLR" -m 'LogError: %q' "could not hardlink $PTH_ESC as ./$BASENAME_ESC"
+                fi
+              done < "${XPLR_PIPE_SELECTION_OUT:?}"
+              echo
+              read -p "[press enter to continue]"
             ]===],
           },
           "PopMode",
@@ -1551,8 +1737,8 @@ xplr.config.modes.builtin.create_file = {
               PTH="$XPLR_INPUT_BUFFER"
               PTH_ESC=$(printf %q "$PTH")
               if [ "$PTH" ]; then
-                mkdir -p -- "$(dirname $(realpath -m $PTH))" \
-                && touch -- "$PTH" \
+                mkdir -p -- "$(dirname $(realpath -m $PTH))"  # This may fail.
+                touch -- "$PTH" \
                 && "$XPLR" -m 'SetInputBuffer: ""' \
                 && "$XPLR" -m 'LogSuccess: %q' "$PTH_ESC created" \
                 && "$XPLR" -m 'ExplorePwd' \
@@ -1618,9 +1804,9 @@ xplr.config.modes.builtin.number = {
 }
 
 xplr.config.modes.builtin.number.key_bindings.on_key["j"] =
-  xplr.config.modes.builtin.number.key_bindings.on_key["down"]
+    xplr.config.modes.builtin.number.key_bindings.on_key["down"]
 xplr.config.modes.builtin.number.key_bindings.on_key["k"] =
-  xplr.config.modes.builtin.number.key_bindings.on_key["up"]
+    xplr.config.modes.builtin.number.key_bindings.on_key["up"]
 
 -- The builtin go to mode.
 --
@@ -1674,13 +1860,13 @@ xplr.config.modes.builtin.go_to = {
                 elif command -v open; then
                   OPENER=open
                 else
-                  "$XPLR" -m 'LogError: "$OPENER not found"'
+                  "$XPLR" -m 'LogError: %q' "$OPENER not found"
                   exit 1
                 fi
               fi
-              (while IFS= read -r -d '' PTH; do
+              while IFS= read -r -d '' PTH; do
                 $OPENER "${PTH:?}" > /dev/null 2>&1
-              done < "${XPLR_PIPE_RESULT_OUT:?}")
+              done < "${XPLR_PIPE_RESULT_OUT:?}"
             ]===],
           },
           "ClearScreen",
@@ -1792,16 +1978,24 @@ xplr.config.modes.builtin.delete = {
         messages = {
           {
             BashExec0 = [===[
-              (while IFS= read -r -d '' PTH; do
+              cat "${XPLR_PIPE_RESULT_OUT:?}" | xargs -0 printf '%q\n'
+              echo
+              read -p "Permanently delete these files? [Y/n]: " ANS
+              [ "${ANS:-Y}" = "Y" ] || [ "$ANS" = "y" ] || exit 0
+              echo
+
+              "$XPLR" -m ExplorePwd
+              while IFS= read -r -d '' PTH; do
                 PTH_ESC=$(printf %q "$PTH")
                 if rm -rfv -- "${PTH:?}"; then
                   "$XPLR" -m 'LogSuccess: %q' "$PTH_ESC deleted"
                 else
-                  "$XPLR" -m 'LogError: %q' "Failed to delete $PTH_ESC"
+                  "$XPLR" -m 'LogError: %q' "could not delete $PTH_ESC"
+                  "$XPLR" -m 'FocusPath: %q' "$PTH"
                 fi
-              done < "${XPLR_PIPE_RESULT_OUT:?}")
-              "$XPLR" -m ExplorePwdAsync
-              read -p "[enter to continue]"
+              done < "${XPLR_PIPE_RESULT_OUT:?}"
+              echo
+              read -p "[press enter to continue]"
             ]===],
           },
           "PopMode",
@@ -1812,24 +2006,33 @@ xplr.config.modes.builtin.delete = {
         messages = {
           {
             BashExec0 = [===[
-              (while IFS= read -r -d '' PTH; do
+              cat "${XPLR_PIPE_RESULT_OUT:?}" | xargs -0 printf '%q\n'
+              echo
+              read -p "Permanently delete these files? [Y/n]: " ANS
+              [ "${ANS:-Y}" = "Y" ] || [ "$ANS" = "y" ] || exit 0
+              echo
+
+              "$XPLR" -m ExplorePwd
+              while IFS= read -r -d '' PTH; do
                 PTH_ESC=$(printf %q "$PTH")
                 if [ -d "$PTH" ] && [ ! -L "$PTH" ]; then
                   if rmdir -v -- "${PTH:?}"; then
                     "$XPLR" -m 'LogSuccess: %q' "$PTH_ESC deleted"
                   else
-                    "$XPLR" -m 'LogError: %q' "Failed to delete $PTH_ESC"
+                    "$XPLR" -m 'LogError: %q' "could not delete $PTH_ESC"
+                    "$XPLR" -m 'FocusPath: %q' "$PTH"
                   fi
                 else
                   if rm -v -- "${PTH:?}"; then
                     "$XPLR" -m 'LogSuccess: %q' "$PTH_ESC deleted"
                   else
-                    "$XPLR" -m 'LogError: %q' "Failed to delete $PTH_ESC"
+                    "$XPLR" -m 'LogError: %q' "could not delete $PTH_ESC"
+                    "$XPLR" -m 'FocusPath: %q' "$PTH"
                   fi
                 fi
-              done < "${XPLR_PIPE_RESULT_OUT:?}")
-              "$XPLR" -m ExplorePwdAsync
-              read -p "[enter to continue]"
+              done < "${XPLR_PIPE_RESULT_OUT:?}"
+              echo
+              read -p "[press enter to continue]"
             ]===],
           },
           "PopMode",
@@ -1899,6 +2102,19 @@ xplr.config.modes.builtin.action = {
           "ToggleMouse",
         },
       },
+      ["p"] = {
+        help = "edit permissions",
+        messages = {
+          "PopMode",
+          { SwitchModeBuiltin = "edit_permissions" },
+          {
+            BashExecSilently0 = [===[
+              PERM=$(stat -c '%a' -- "${XPLR_FOCUS_PATH:?}")
+              "$XPLR" -m 'SetInputBuffer: %q' "${PERM:?}"
+            ]===],
+          },
+        },
+      },
       ["v"] = {
         help = "vroot",
         messages = {
@@ -1987,6 +2203,42 @@ xplr.config.modes.builtin.search = {
           "FocusNext",
         },
       },
+      ["ctrl-z"] = {
+        help = "toggle ordering",
+        messages = {
+          "ToggleSearchOrder",
+          "ExplorePwdAsync",
+        },
+      },
+      ["ctrl-a"] = {
+        help = "toggle search algorithm",
+        messages = {
+          "ToggleSearchAlgorithm",
+          "ExplorePwdAsync",
+        },
+      },
+      ["ctrl-r"] = {
+        help = "regex search",
+        messages = {
+          "SearchRegexFromInput",
+          "ExplorePwdAsync",
+        },
+      },
+      ["ctrl-f"] = {
+        help = "fuzzy search",
+        messages = {
+          "SearchFuzzyFromInput",
+          "ExplorePwdAsync",
+        },
+      },
+      ["ctrl-s"] = {
+        help = "sort (no search order)",
+        messages = {
+          "DisableSearchOrder",
+          "ExplorePwdAsync",
+          { SwitchModeBuiltinKeepingInputBuffer = "sort" },
+        },
+      },
       ["right"] = {
         help = "enter",
         messages = {
@@ -2026,7 +2278,7 @@ xplr.config.modes.builtin.search = {
     default = {
       messages = {
         "UpdateInputBufferFromKey",
-        "SearchFuzzyFromInput",
+        "SearchFromInput",
         "ExplorePwdAsync",
       },
     },
@@ -2034,9 +2286,9 @@ xplr.config.modes.builtin.search = {
 }
 
 xplr.config.modes.builtin.search.key_bindings.on_key["ctrl-n"] =
-  xplr.config.modes.builtin.search.key_bindings.on_key["down"]
+    xplr.config.modes.builtin.search.key_bindings.on_key["down"]
 xplr.config.modes.builtin.search.key_bindings.on_key["ctrl-p"] =
-  xplr.config.modes.builtin.search.key_bindings.on_key["up"]
+    xplr.config.modes.builtin.search.key_bindings.on_key["up"]
 
 -- The builtin filter mode.
 --
@@ -2236,7 +2488,12 @@ xplr.config.modes.builtin.sort = {
       ["enter"] = {
         help = "submit",
         messages = {
-          "PopMode",
+          "PopModeKeepingInputBuffer",
+        },
+      },
+      ["esc"] = {
+        messages = {
+          "PopModeKeepingInputBuffer",
         },
       },
       ["m"] = {
@@ -2269,7 +2526,6 @@ xplr.config.modes.builtin.sort = {
           "ExplorePwdAsync",
         },
       },
-
       ["c"] = {
         help = "by created",
         messages = {
@@ -2277,7 +2533,6 @@ xplr.config.modes.builtin.sort = {
           "ExplorePwdAsync",
         },
       },
-
       ["C"] = {
         help = "by created reverse",
         messages = {
@@ -2285,7 +2540,6 @@ xplr.config.modes.builtin.sort = {
           "ExplorePwdAsync",
         },
       },
-
       ["l"] = {
         help = "by last modified",
         messages = {
@@ -2293,7 +2547,6 @@ xplr.config.modes.builtin.sort = {
           "ExplorePwdAsync",
         },
       },
-
       ["L"] = {
         help = "by last modified reverse",
         messages = {
@@ -2407,6 +2660,161 @@ xplr.config.modes.builtin.vroot = {
   },
 }
 
+-- The builtin edit permissions mode.
+--
+-- Type: [Mode](https://xplr.dev/en/mode)
+xplr.config.modes.builtin.edit_permissions = {
+  name = "edit permissions",
+  key_bindings = {
+    on_key = {
+      ["u"] = {
+        help = "+user",
+        messages = {
+          {
+            BashExecSilently0 = [===[
+              PERM="${XPLR_INPUT_BUFFER:-000}"
+              U="${PERM: -3:-2}"
+              G="${PERM: -2:-1}"
+              O="${PERM: -1}"
+
+              U="$(( (${U:-0} + 1) % 8 ))"
+              "$XPLR" -m 'SetInputBuffer: %q' "${U:-0}${G:-0}${O:-0}"
+            ]===],
+          },
+        },
+      },
+      ["U"] = {
+        help = "-user",
+        messages = {
+          {
+            BashExecSilently0 = [===[
+              PERM="${XPLR_INPUT_BUFFER:-000}"
+              U="${PERM: -3:-2}"
+              G="${PERM: -2:-1}"
+              O="${PERM: -1}"
+
+              U="$(( ${U:-0}-1 < 0 ? 7 : ${U:-0}-1 ))"
+              "$XPLR" -m 'SetInputBuffer: %q' "${U:-0}${G:-0}${O:-0}"
+            ]===],
+          },
+        },
+      },
+      ["g"] = {
+        help = "+group",
+        messages = {
+          {
+            BashExecSilently0 = [===[
+              PERM="${XPLR_INPUT_BUFFER:-000}"
+              U="${PERM: -3:-2}"
+              G="${PERM: -2:-1}"
+              O="${PERM: -1}"
+
+              G="$(( (${G:-0} + 1) % 8 ))"
+              "$XPLR" -m 'SetInputBuffer: %q' "${U:-0}${G:-0}${O:-0}"
+            ]===],
+          },
+        },
+      },
+      ["G"] = {
+        help = "-group",
+        messages = {
+          {
+            BashExecSilently0 = [===[
+              PERM="${XPLR_INPUT_BUFFER:-000}"
+              U="${PERM: -3:-2}"
+              G="${PERM: -2:-1}"
+              O="${PERM: -1}"
+
+              G="$(( ${G:-0}-1 < 0 ? 7 : ${G:-0}-1 ))"
+              "$XPLR" -m 'SetInputBuffer: %q' "${U:-0}${G:-0}${O:-0}"
+            ]===],
+          },
+        },
+      },
+      ["o"] = {
+        help = "+other",
+        messages = {
+          {
+            BashExecSilently0 = [===[
+              PERM="${XPLR_INPUT_BUFFER:-000}"
+              U="${PERM: -3:-2}"
+              G="${PERM: -2:-1}"
+              O="${PERM: -1}"
+
+              O="$(( (${O:-0} + 1) % 8 ))"
+              "$XPLR" -m 'SetInputBuffer: %q' "${U:-0}${G:-0}${O:-0}"
+            ]===],
+          },
+        },
+      },
+      ["O"] = {
+        help = "-other",
+        messages = {
+          {
+            BashExecSilently0 = [===[
+              PERM="${XPLR_INPUT_BUFFER:-000}"
+              U="${PERM: -3:-2}"
+              G="${PERM: -2:-1}"
+              O="${PERM: -1}"
+
+              O="$(( ${O:-0}-1 < 0 ? 7 : ${O:-0}-1 ))"
+              "$XPLR" -m 'SetInputBuffer: %q' "${U:-0}${G:-0}${O:-0}"
+            ]===],
+          },
+        },
+      },
+      ["m"] = {
+        help = "max",
+        messages = {
+          {
+            BashExecSilently0 = [===[
+              "$XPLR" -m 'SetInputBuffer: %q' "777"
+            ]===],
+          },
+        },
+      },
+      ["M"] = {
+        help = "min",
+        messages = {
+          {
+            BashExecSilently0 = [===[
+              "$XPLR" -m 'SetInputBuffer: %q' "000"
+            ]===],
+          },
+        },
+      },
+      ["ctrl-r"] = {
+        help = "reset",
+        messages = {
+          {
+            BashExecSilently0 = [===[
+              PERM=$(stat -c '%a' -- "${XPLR_FOCUS_PATH:?}")
+              "$XPLR" -m 'SetInputBuffer: %q' "${PERM:?}"
+            ]===],
+          },
+        },
+      },
+      ["enter"] = {
+        help = "submit",
+        messages = {
+          {
+            BashExecSilently0 = [===[
+              chmod "${XPLR_INPUT_BUFFER:?}" -- "${XPLR_FOCUS_PATH:?}"
+            ]===],
+          },
+          "PopMode",
+          "ExplorePwdAsync",
+        },
+      },
+    },
+    default = {
+      messages = {
+        "UpdateInputBufferFromKey",
+      },
+    },
+  },
+}
+
 -- This is where you define custom modes.
 --
 -- Type: mapping of the following key-value pairs:
@@ -2505,6 +2913,19 @@ xplr.fn.builtin.try_complete_path = function(m)
   end
 end
 
+xplr.fn.builtin.fmt_general_selection_item = function(n)
+  local nl = xplr.util.paint("\\n", { add_modifiers = { "Italic", "Dim" } })
+  local sh_config = { with_prefix_dots = true, without_suffix_dots = true }
+  local shortened = xplr.util.shorten(n.absolute_path, sh_config)
+  if n.is_dir then
+    shortened = shortened .. "/"
+  end
+  local ls_style = xplr.util.lscolor(n.absolute_path)
+  local meta_style = xplr.util.node_type(n).style
+  local style = xplr.util.style_mix({ ls_style, meta_style })
+  return xplr.util.paint(shortened:gsub("\n", nl), style)
+end
+
 -- Renders the first column in the table
 xplr.fn.builtin.fmt_general_table_row_cols_0 = function(m)
   local r = ""
@@ -2521,11 +2942,10 @@ end
 
 -- Renders the second column in the table
 xplr.fn.builtin.fmt_general_table_row_cols_1 = function(m)
+  local nl = xplr.util.paint("\\n", { add_modifiers = { "Italic", "Dim" } })
   local r = m.tree .. m.prefix
-
-  local function path_escape(path)
-    return string.gsub(string.gsub(path, "\\", "\\\\"), "\n", "\\n")
-  end
+  local style = xplr.util.lscolor(m.absolute_path)
+  style = xplr.util.style_mix({ style, m.style })
 
   if m.meta.icon == nil then
     r = r .. ""
@@ -2533,11 +2953,11 @@ xplr.fn.builtin.fmt_general_table_row_cols_1 = function(m)
     r = r .. m.meta.icon .. " "
   end
 
-  r = r .. path_escape(m.relative_path)
-
+  local rel = m.relative_path
   if m.is_dir then
-    r = r .. "/"
+    rel = rel .. "/"
   end
+  r = r .. xplr.util.paint(xplr.util.shell_escape(rel), style)
 
   r = r .. m.suffix .. " "
 
@@ -2547,11 +2967,11 @@ xplr.fn.builtin.fmt_general_table_row_cols_1 = function(m)
     if m.is_broken then
       r = r .. "×"
     else
-      r = r .. path_escape(m.symlink.absolute_path)
-
+      local symlink_path = xplr.util.shorten(m.symlink.absolute_path)
       if m.symlink.is_dir then
-        r = r .. "/"
+        symlink_path = symlink_path .. "/"
       end
+      r = r .. symlink_path:gsub("\n", nl)
     end
   end
 
@@ -2560,84 +2980,23 @@ end
 
 -- Renders the third column in the table
 xplr.fn.builtin.fmt_general_table_row_cols_2 = function(m)
-  local no_color = os.getenv("NO_COLOR")
+  local r = xplr.util.paint("r", { fg = "Green" })
+  local w = xplr.util.paint("w", { fg = "Yellow" })
+  local x = xplr.util.paint("x", { fg = "Red" })
+  local s = xplr.util.paint("s", { fg = "Red" })
+  local S = xplr.util.paint("S", { fg = "Red" })
+  local t = xplr.util.paint("t", { fg = "Red" })
+  local T = xplr.util.paint("T", { fg = "Red" })
 
-  local function green(x)
-    if no_color == nil then
-      return "\x1b[32m" .. x .. "\x1b[0m"
-    else
-      return x
-    end
-  end
-
-  local function yellow(x)
-    if no_color == nil then
-      return "\x1b[33m" .. x .. "\x1b[0m"
-    else
-      return x
-    end
-  end
-
-  local function red(x)
-    if no_color == nil then
-      return "\x1b[31m" .. x .. "\x1b[0m"
-    else
-      return x
-    end
-  end
-
-  local function bit(x, color, cond)
-    if cond then
-      return color(x)
-    else
-      return color("-")
-    end
-  end
-
-  local p = m.permissions
-
-  local r = ""
-
-  r = r .. bit("r", green, p.user_read)
-  r = r .. bit("w", yellow, p.user_write)
-
-  if p.user_execute == false and p.setuid == false then
-    r = r .. bit("-", red, p.user_execute)
-  elseif p.user_execute == true and p.setuid == false then
-    r = r .. bit("x", red, p.user_execute)
-  elseif p.user_execute == false and p.setuid == true then
-    r = r .. bit("S", red, p.user_execute)
-  else
-    r = r .. bit("s", red, p.user_execute)
-  end
-
-  r = r .. bit("r", green, p.group_read)
-  r = r .. bit("w", yellow, p.group_write)
-
-  if p.group_execute == false and p.setuid == false then
-    r = r .. bit("-", red, p.group_execute)
-  elseif p.group_execute == true and p.setuid == false then
-    r = r .. bit("x", red, p.group_execute)
-  elseif p.group_execute == false and p.setuid == true then
-    r = r .. bit("S", red, p.group_execute)
-  else
-    r = r .. bit("s", red, p.group_execute)
-  end
-
-  r = r .. bit("r", green, p.other_read)
-  r = r .. bit("w", yellow, p.other_write)
-
-  if p.other_execute == false and p.setuid == false then
-    r = r .. bit("-", red, p.other_execute)
-  elseif p.other_execute == true and p.setuid == false then
-    r = r .. bit("x", red, p.other_execute)
-  elseif p.other_execute == false and p.setuid == true then
-    r = r .. bit("T", red, p.other_execute)
-  else
-    r = r .. bit("t", red, p.other_execute)
-  end
-
-  return r
+  return xplr.util
+      .permissions_rwx(m.permissions)
+      :gsub("r", r)
+      :gsub("w", w)
+      :gsub("x", x)
+      :gsub("s", s)
+      :gsub("S", S)
+      :gsub("t", t)
+      :gsub("T", T)
 end
 
 -- Renders the fourth column in the table
@@ -2727,3 +3086,15 @@ return {
   on_mode_switch = {},
   on_layout_switch = {},
 }
+
+-- ----------------------------------------------------------------------------
+-- > Note:
+-- >
+-- > It's not recommended to copy the entire configuration, unless you want to
+-- > freeze it and miss out on useful updates to the defaults.
+-- >
+-- > Instead, you can use this as a reference to overwrite only the parts you
+-- > want to update.
+-- >
+-- > If you still want to copy the entire configuration, make sure to put your
+-- > customization before the return statement.
